@@ -14,19 +14,20 @@
  ***************************************************************************/
 
 #include "qgsterraingenerator.h"
+#include "moc_qgsterraingenerator.cpp"
 
-#include "qgsaabb.h"
 #include "qgs3dmapsettings.h"
 #include "qgs3dutils.h"
 #include "qgscoordinatetransform.h"
+#include "qgsabstractterrainsettings.h"
 
-QgsAABB QgsTerrainGenerator::rootChunkBbox( const Qgs3DMapSettings &map ) const
+QgsBox3D QgsTerrainGenerator::rootChunkBox3D( const Qgs3DMapSettings &map ) const
 {
   QgsRectangle te = Qgs3DUtils::tryReprojectExtent2D( rootChunkExtent(), crs(), map.crs(), map.transformContext() );
 
   float hMin, hMax;
   rootChunkHeightRange( hMin, hMax );
-  return Qgs3DUtils::mapToWorldExtent( te, hMin * map.terrainVerticalScale(), hMax * map.terrainVerticalScale(), map.origin() );
+  return QgsBox3D( te.xMinimum(), te.yMinimum(), hMin * map.terrainSettings()->verticalScale(), te.xMaximum(), te.yMaximum(), hMax * map.terrainSettings()->verticalScale() );
 }
 
 float QgsTerrainGenerator::rootChunkError( const Qgs3DMapSettings &map ) const
@@ -34,7 +35,7 @@ float QgsTerrainGenerator::rootChunkError( const Qgs3DMapSettings &map ) const
   QgsRectangle te = Qgs3DUtils::tryReprojectExtent2D( rootChunkExtent(), crs(), map.crs(), map.transformContext() );
 
   // use texel size as the error
-  return te.width() / map.mapTileResolution();
+  return te.width() / map.terrainSettings()->mapTileResolution();
 }
 
 void QgsTerrainGenerator::rootChunkHeightRange( float &hMin, float &hMax ) const
@@ -70,6 +71,10 @@ QString QgsTerrainGenerator::typeToString( QgsTerrainGenerator::Type type )
   return QString();
 }
 
+void QgsTerrainGenerator::setCrs( const QgsCoordinateReferenceSystem &, const QgsCoordinateTransformContext & )
+{
+}
+
 bool QgsTerrainGenerator::isValid() const
 {
   return mIsValid;
