@@ -78,7 +78,11 @@ QString QgsGridAlgorithm::shortHelpString() const
                       "That means that, at that point, an element is guaranteed to be placed. "
                       "Unless the width and height of the selected extent is a multiple of the "
                       "selected spacing, that is not true for the other points that define that extent."
-                    );
+  );
+}
+QString QgsGridAlgorithm::shortDescription() const
+{
+  return QObject::tr( "Creates a vector layer with a grid covering a given extent." );
 }
 
 QgsGridAlgorithm *QgsGridAlgorithm::createInstance() const
@@ -145,7 +149,7 @@ QVariantMap QgsGridAlgorithm::processAlgorithm( const QVariantMap &parameters, Q
   }
 
   QString dest;
-  std::unique_ptr< QgsFeatureSink > sink( parameterAsSink( parameters, QStringLiteral( "OUTPUT" ), context, dest, fields, outputWkb, mCrs ) );
+  std::unique_ptr<QgsFeatureSink> sink( parameterAsSink( parameters, QStringLiteral( "OUTPUT" ), context, dest, fields, outputWkb, mCrs ) );
   if ( !sink )
     throw QgsProcessingException( invalidSinkError( parameters, QStringLiteral( "OUTPUT" ) ) );
 
@@ -170,13 +174,13 @@ QVariantMap QgsGridAlgorithm::processAlgorithm( const QVariantMap &parameters, Q
       break;
   }
 
-
+  sink->finalize();
   QVariantMap outputs;
   outputs.insert( QStringLiteral( "OUTPUT" ), dest );
   return outputs;
 }
 
-void QgsGridAlgorithm::createPointGrid( std::unique_ptr< QgsFeatureSink > &sink, QgsProcessingFeedback *feedback )
+void QgsGridAlgorithm::createPointGrid( std::unique_ptr<QgsFeatureSink> &sink, QgsProcessingFeedback *feedback )
 {
   QgsFeature f = QgsFeature();
 
@@ -221,7 +225,7 @@ void QgsGridAlgorithm::createPointGrid( std::unique_ptr< QgsFeatureSink > &sink,
   }
 }
 
-void QgsGridAlgorithm::createLineGrid( std::unique_ptr< QgsFeatureSink > &sink, QgsProcessingFeedback *feedback )
+void QgsGridAlgorithm::createLineGrid( std::unique_ptr<QgsFeatureSink> &sink, QgsProcessingFeedback *feedback )
 {
   QgsFeature f = QgsFeature();
 
@@ -284,7 +288,6 @@ void QgsGridAlgorithm::createLineGrid( std::unique_ptr< QgsFeatureSink > &sink,
       lastProgress = thisProgress;
       feedback->setProgress( lastProgress );
     }
-
   }
   //set progress to 50 manually in case the division doesn't amount to 50.
   feedback->setProgress( 50 );
@@ -325,7 +328,7 @@ void QgsGridAlgorithm::createLineGrid( std::unique_ptr< QgsFeatureSink > &sink,
   feedback->setProgress( 100 );
 }
 
-void QgsGridAlgorithm::createRectangleGrid( std::unique_ptr< QgsFeatureSink > &sink, QgsProcessingFeedback *feedback )
+void QgsGridAlgorithm::createRectangleGrid( std::unique_ptr<QgsFeatureSink> &sink, QgsProcessingFeedback *feedback )
 {
   QgsFeature f = QgsFeature();
 
@@ -338,8 +341,8 @@ void QgsGridAlgorithm::createRectangleGrid( std::unique_ptr< QgsFeatureSink > &s
 
   int thisProgress = 0;
   int lastProgress = 0;
-  QVector< double > ringX( 5 );
-  QVector< double > ringY( 5 );
+  QVector<double> ringX( 5 );
+  QVector<double> ringY( 5 );
 
   for ( long long col = 0; col < cols; col++ )
   {
@@ -356,7 +359,7 @@ void QgsGridAlgorithm::createRectangleGrid( std::unique_ptr< QgsFeatureSink > &s
 
       ringX = { x1, x2, x2, x1, x1 };
       ringY = { y1, y1, y2, y2, y1 };
-      std::unique_ptr< QgsPolygon > poly = std::make_unique< QgsPolygon >();
+      auto poly = std::make_unique<QgsPolygon>();
       poly->setExteriorRing( new QgsLineString( ringX, ringY ) );
       f.setGeometry( std::move( poly ) );
       f.setAttributes( QgsAttributes() << id << x1 << y1 << x2 << y2 << row << col );
@@ -379,7 +382,7 @@ void QgsGridAlgorithm::createRectangleGrid( std::unique_ptr< QgsFeatureSink > &s
   }
 }
 
-void QgsGridAlgorithm::createDiamondGrid( std::unique_ptr< QgsFeatureSink > &sink, QgsProcessingFeedback *feedback )
+void QgsGridAlgorithm::createDiamondGrid( std::unique_ptr<QgsFeatureSink> &sink, QgsProcessingFeedback *feedback )
 {
   QgsFeature f = QgsFeature();
 
@@ -389,7 +392,7 @@ void QgsGridAlgorithm::createDiamondGrid( std::unique_ptr< QgsFeatureSink > &sin
   const double halfHOverlay = mHOverlay / 2;
   const double halfVOverlay = mVOverlay / 2;
 
-  const long long cols =  static_cast<long long>( std::ceil( mGridExtent.width() / ( halfHSpacing - halfHOverlay ) ) );
+  const long long cols = static_cast<long long>( std::ceil( mGridExtent.width() / ( halfHSpacing - halfHOverlay ) ) );
   const long long rows = static_cast<long long>( std::ceil( mGridExtent.height() / ( mVSpacing - halfVOverlay ) ) );
 
   long long id = 1;
@@ -398,8 +401,8 @@ void QgsGridAlgorithm::createDiamondGrid( std::unique_ptr< QgsFeatureSink > &sin
 
   int thisProgress = 0;
   int lastProgress = 0;
-  QVector< double > ringX( 5 );
-  QVector< double > ringY( 5 );
+  QVector<double> ringX( 5 );
+  QVector<double> ringY( 5 );
 
   for ( long long col = 0; col < cols; col++ )
   {
@@ -434,7 +437,7 @@ void QgsGridAlgorithm::createDiamondGrid( std::unique_ptr< QgsFeatureSink > &sin
 
       ringX = { x1, x2, x3, x2, x1 };
       ringY = { y2, y1, y2, y3, y2 };
-      std::unique_ptr< QgsPolygon > poly = std::make_unique< QgsPolygon >();
+      auto poly = std::make_unique<QgsPolygon>();
       poly->setExteriorRing( new QgsLineString( ringX, ringY ) );
       f.setGeometry( std::move( poly ) );
       f.setAttributes( QgsAttributes() << id << x1 << y1 << x3 << y3 );
@@ -476,7 +479,7 @@ void QgsGridAlgorithm::createHexagonGrid( std::unique_ptr<QgsFeatureSink> &sink,
 
   const double halfVSpacing = mVSpacing / 2;
 
-  const long long cols =  static_cast<long long>( std::ceil( mGridExtent.width() / ( mHOverlay ) ) );
+  const long long cols = static_cast<long long>( std::ceil( mGridExtent.width() / ( mHOverlay ) ) );
   const long long rows = static_cast<long long>( std::ceil( mGridExtent.height() / ( mVSpacing - mVOverlay ) ) );
 
   long long id = 1;
@@ -486,8 +489,8 @@ void QgsGridAlgorithm::createHexagonGrid( std::unique_ptr<QgsFeatureSink> &sink,
   int thisProgress = 0;
   int lastProgress = 0;
 
-  QVector< double > ringX( 7 );
-  QVector< double > ringY( 7 );
+  QVector<double> ringX( 7 );
+  QVector<double> ringY( 7 );
   for ( long long col = 0; col < cols; col++ )
   {
     if ( feedback && feedback->isCanceled() )
@@ -523,7 +526,7 @@ void QgsGridAlgorithm::createHexagonGrid( std::unique_ptr<QgsFeatureSink> &sink,
 
       ringX = { x1, x2, x3, x4, x3, x2, x1 };
       ringY = { y2, y1, y1, y2, y3, y3, y2 };
-      std::unique_ptr< QgsPolygon > poly = std::make_unique< QgsPolygon >();
+      auto poly = std::make_unique<QgsPolygon>();
       poly->setExteriorRing( new QgsLineString( ringX, ringY ) );
       f.setGeometry( std::move( poly ) );
       f.setAttributes( QgsAttributes() << id << x1 << y1 << x4 << y3 << row << col );
