@@ -17,6 +17,7 @@
 
 #include "qgsproviderregistry.h"
 #include "qgsrasterdataprovider.h"
+#include "moc_qgsrasterdataprovider.cpp"
 #include "qgsrasteridentifyresult.h"
 #include "qgslogger.h"
 #include "qgspoint.h"
@@ -54,7 +55,7 @@ QgsRasterBlock *QgsRasterDataProvider::block( int bandNo, QgsRectangle  const &b
   QgsDebugMsgLevel( QStringLiteral( "bandNo = %1 width = %2 height = %3" ).arg( bandNo ).arg( width ).arg( height ), 4 );
   QgsDebugMsgLevel( QStringLiteral( "boundingBox = %1" ).arg( boundingBox.toString() ), 4 );
 
-  std::unique_ptr< QgsRasterBlock > block = std::make_unique< QgsRasterBlock >( dataType( bandNo ), width, height );
+  auto block = std::make_unique< QgsRasterBlock >( dataType( bandNo ), width, height );
   if ( sourceHasNoDataValue( bandNo ) && useSourceNoDataValue( bandNo ) )
   {
     block->setNoDataValue( sourceNoDataValue( bandNo ) );
@@ -153,7 +154,7 @@ QgsRasterBlock *QgsRasterDataProvider::block( int bandNo, QgsRectangle  const &b
     QgsDebugMsgLevel( QStringLiteral( "Reading smaller block tmpWidth = %1 height = %2" ).arg( tmpWidth ).arg( tmpHeight ), 4 );
     QgsDebugMsgLevel( QStringLiteral( "tmpExtent = %1" ).arg( tmpExtent.toString() ), 4 );
 
-    std::unique_ptr< QgsRasterBlock > tmpBlock = std::make_unique< QgsRasterBlock >( dataType( bandNo ), tmpWidth, tmpHeight );
+    auto tmpBlock = std::make_unique< QgsRasterBlock >( dataType( bandNo ), tmpWidth, tmpHeight );
     if ( sourceHasNoDataValue( bandNo ) && useSourceNoDataValue( bandNo ) )
     {
       tmpBlock->setNoDataValue( sourceNoDataValue( bandNo ) );
@@ -369,6 +370,7 @@ bool QgsRasterDataProvider::writeBlock( QgsRasterBlock *block, int band, int xOf
     return false;
   if ( !isEditable() )
   {
+    setError( QgsError( QStringLiteral( "writeBlock() called on read-only provider." ), QStringLiteral( "writeBlock" ) ) );
     QgsDebugError( QStringLiteral( "writeBlock() called on read-only provider." ) );
     return false;
   }
@@ -456,13 +458,13 @@ QgsRasterDataProvider *QgsRasterDataProvider::create( const QString &providerKey
     Qgis::DataType type,
     int width, int height, double *geoTransform,
     const QgsCoordinateReferenceSystem &crs,
-    const QStringList &createOptions )
+    const QStringList &creationOptions )
 {
   QgsRasterDataProvider *ret = QgsProviderRegistry::instance()->createRasterDataProvider(
                                  providerKey,
                                  uri, format,
                                  nBands, type, width,
-                                 height, geoTransform, crs, createOptions );
+                                 height, geoTransform, crs, creationOptions );
   if ( !ret )
   {
     QgsDebugError( "Cannot resolve 'createRasterDataProviderFunction' function in " + providerKey + " provider" );
@@ -597,37 +599,37 @@ void QgsRasterDataProvider::copyBaseSettings( const QgsRasterDataProvider &other
   }
 }
 
-static QgsRasterDataProvider::ResamplingMethod resamplingMethodFromString( const QString &str )
+static Qgis::RasterResamplingMethod resamplingMethodFromString( const QString &str )
 {
   if ( str == QLatin1String( "bilinear" ) )
   {
-    return QgsRasterDataProvider::ResamplingMethod::Bilinear;
+    return Qgis::RasterResamplingMethod::Bilinear;
   }
   else if ( str == QLatin1String( "cubic" ) )
   {
-    return QgsRasterDataProvider::ResamplingMethod::Cubic;
+    return Qgis::RasterResamplingMethod::Cubic;
   }
   else if ( str == QLatin1String( "cubicSpline" ) )
   {
-    return QgsRasterDataProvider::ResamplingMethod::CubicSpline;
+    return Qgis::RasterResamplingMethod::CubicSpline;
   }
   else if ( str == QLatin1String( "lanczos" ) )
   {
-    return QgsRasterDataProvider::ResamplingMethod::Lanczos;
+    return Qgis::RasterResamplingMethod::Lanczos;
   }
   else if ( str == QLatin1String( "average" ) )
   {
-    return QgsRasterDataProvider::ResamplingMethod::Average;
+    return Qgis::RasterResamplingMethod::Average;
   }
   else if ( str == QLatin1String( "mode" ) )
   {
-    return QgsRasterDataProvider::ResamplingMethod::Mode;
+    return Qgis::RasterResamplingMethod::Mode;
   }
   else if ( str == QLatin1String( "gauss" ) )
   {
-    return QgsRasterDataProvider::ResamplingMethod::Gauss;
+    return Qgis::RasterResamplingMethod::Gauss;
   }
-  return  QgsRasterDataProvider::ResamplingMethod::Nearest;
+  return  Qgis::RasterResamplingMethod::Nearest;
 }
 
 void QgsRasterDataProvider::readXml( const QDomElement &filterElem )
@@ -649,25 +651,25 @@ void QgsRasterDataProvider::readXml( const QDomElement &filterElem )
   }
 }
 
-static QString resamplingMethodToString( QgsRasterDataProvider::ResamplingMethod method )
+static QString resamplingMethodToString( Qgis::RasterResamplingMethod method )
 {
   switch ( method )
   {
-    case QgsRasterDataProvider::ResamplingMethod::Nearest:
+    case Qgis::RasterResamplingMethod::Nearest:
       return QStringLiteral( "nearestNeighbour" );
-    case QgsRasterDataProvider::ResamplingMethod::Bilinear:
+    case Qgis::RasterResamplingMethod::Bilinear:
       return QStringLiteral( "bilinear" );
-    case QgsRasterDataProvider::ResamplingMethod::Cubic:
+    case Qgis::RasterResamplingMethod::Cubic:
       return QStringLiteral( "cubic" );
-    case QgsRasterDataProvider::ResamplingMethod::CubicSpline:
+    case Qgis::RasterResamplingMethod::CubicSpline:
       return QStringLiteral( "cubicSpline" );
-    case QgsRasterDataProvider::ResamplingMethod::Lanczos:
+    case Qgis::RasterResamplingMethod::Lanczos:
       return QStringLiteral( "lanczos" );
-    case QgsRasterDataProvider::ResamplingMethod::Average:
+    case Qgis::RasterResamplingMethod::Average:
       return QStringLiteral( "average" );
-    case QgsRasterDataProvider::ResamplingMethod::Mode:
+    case Qgis::RasterResamplingMethod::Mode:
       return QStringLiteral( "mode" );
-    case QgsRasterDataProvider::ResamplingMethod::Gauss:
+    case Qgis::RasterResamplingMethod::Gauss:
       return QStringLiteral( "gauss" );
   }
   // should not happen
@@ -773,7 +775,7 @@ bool QgsRasterDataProvider::readFileBasedAttributeTable( int bandNumber, const Q
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
 
-  std::unique_ptr<QgsRasterAttributeTable> rat = std::make_unique<QgsRasterAttributeTable>();
+  auto rat = std::make_unique<QgsRasterAttributeTable>();
   if ( rat->readFromFile( path, errorMessage ) )
   {
     setAttributeTable( bandNumber, rat.release() );
@@ -1049,3 +1051,5 @@ QString QgsRasterDataProvider::encodeVirtualRasterProviderUri( const VirtualRast
   uri.setQuery( query );
   return QString( QUrl::toPercentEncoding( uri.toEncoded() ) );
 }
+
+#undef ERR
