@@ -18,6 +18,7 @@
 
 #include "qgsapplication.h"
 #include "qgselevationprofilecanvas.h"
+#include "moc_qgselevationprofilecanvas.cpp"
 #include "qgsmaplayerlistutils_p.h"
 #include "qgsplotcanvasitem.h"
 #include "qgsprofilerequest.h"
@@ -36,7 +37,6 @@
 #include "qgsmaplayerelevationproperties.h"
 #include "qgsscreenhelper.h"
 #include "qgsfillsymbol.h"
-#include "qgslinesymbol.h"
 #include "qgsprofilesourceregistry.h"
 
 #include <QWheelEvent>
@@ -47,7 +47,6 @@
 class QgsElevationProfilePlotItem : public Qgs2DPlot, public QgsPlotCanvasItem
 {
   public:
-
     QgsElevationProfilePlotItem( QgsElevationProfileCanvas *canvas )
       : QgsPlotCanvasItem( canvas )
     {
@@ -197,7 +196,7 @@ class QgsElevationProfilePlotItem : public Qgs2DPlot, public QgsPlotCanvasItem
 
     QgsPointXY plotPointToCanvasPoint( const QgsProfilePoint &point )
     {
-      if ( point.distance() < xMinimum() * mXScaleFactor || point.distance() > xMaximum()* mXScaleFactor || point.elevation() < yMinimum() || point.elevation() > yMaximum() )
+      if ( point.distance() < xMinimum() * mXScaleFactor || point.distance() > xMaximum() * mXScaleFactor || point.elevation() < yMinimum() || point.elevation() > yMaximum() )
         return QgsPointXY();
 
       const QRectF area = plotArea();
@@ -227,13 +226,11 @@ class QgsElevationProfilePlotItem : public Qgs2DPlot, public QgsPlotCanvasItem
         }
         else
         {
-          plot = mRenderer->renderToImage( plotArea.width() * pixelRatio,
-                                           plotArea.height() * pixelRatio, xMinimum() * mXScaleFactor, xMaximum() * mXScaleFactor, yMinimum(), yMaximum(), source, pixelRatio );
+          plot = mRenderer->renderToImage( plotArea.width() * pixelRatio, plotArea.height() * pixelRatio, xMinimum() * mXScaleFactor, xMaximum() * mXScaleFactor, yMinimum(), yMaximum(), source, pixelRatio );
           plot.setDevicePixelRatio( pixelRatio );
           mCachedImages.insert( source, plot );
         }
-        rc.painter()->drawImage( QPointF( plotArea.left(),
-                                          plotArea.top() ), plot );
+        rc.painter()->drawImage( QPointF( plotArea.left(), plotArea.top() ), plot );
       }
     }
 
@@ -270,16 +267,24 @@ class QgsElevationProfilePlotItem : public Qgs2DPlot, public QgsPlotCanvasItem
       }
     }
 
+    void setSubsectionsSymbol( QgsLineSymbol *symbol )
+    {
+      if ( mRenderer )
+      {
+        mRenderer->setSubsectionsSymbol( symbol );
+        updatePlot();
+      }
+    }
+
     QgsProject *mProject = nullptr;
     double mXScaleFactor = 1.0;
 
     Qgis::DistanceUnit mDistanceUnit = Qgis::DistanceUnit::Unknown;
 
   private:
-
     QImage mImage;
 
-    QMap< QString, QImage > mCachedImages;
+    QMap<QString, QImage> mCachedImages;
 
     QRectF mRect;
     QRectF mPlotArea;
@@ -289,7 +294,6 @@ class QgsElevationProfilePlotItem : public Qgs2DPlot, public QgsPlotCanvasItem
 class QgsElevationProfileCrossHairsItem : public QgsPlotCanvasItem
 {
   public:
-
     QgsElevationProfileCrossHairsItem( QgsElevationProfileCanvas *canvas, QgsElevationProfilePlotItem *plotItem )
       : QgsPlotCanvasItem( canvas )
       , mPlotItem( plotItem )
@@ -318,7 +322,7 @@ class QgsElevationProfileCrossHairsItem : public QgsPlotCanvasItem
 
     void paint( QPainter *painter ) override
     {
-      const QgsPointXY crossHairPlotPoint  = mPlotItem->plotPointToCanvasPoint( mPoint );
+      const QgsPointXY crossHairPlotPoint = mPlotItem->plotPointToCanvasPoint( mPoint );
       if ( crossHairPlotPoint.isEmpty() )
         return;
 
@@ -407,7 +411,6 @@ class QgsElevationProfileCrossHairsItem : public QgsPlotCanvasItem
     }
 
   private:
-
     QRectF mRect;
     QgsProfilePoint mPoint;
     QgsElevationProfilePlotItem *mPlotItem = nullptr;
@@ -439,7 +442,6 @@ QgsElevationProfileCanvas::QgsElevationProfileCanvas( QWidget *parent )
   mDeferredRedrawTimer->setSingleShot( true );
   mDeferredRedrawTimer->stop();
   connect( mDeferredRedrawTimer, &QTimer::timeout, this, &QgsElevationProfileCanvas::startDeferredRedraw );
-
 }
 
 QgsElevationProfileCanvas::~QgsElevationProfileCanvas()
@@ -470,7 +472,7 @@ void QgsElevationProfileCanvas::panContentsBy( double dx, double dy )
   const double dyPercent = dy / mPlotItem->plotArea().height();
 
   // these look backwards, but we are dragging the paper, not the view!
-  const double dxPlot = - dxPercent * ( mPlotItem->xMaximum() - mPlotItem->xMinimum() );
+  const double dxPlot = -dxPercent * ( mPlotItem->xMaximum() - mPlotItem->xMinimum() );
   const double dyPlot = dyPercent * ( mPlotItem->yMaximum() - mPlotItem->yMinimum() );
 
   // no need to handle axis scale lock here, we aren't changing scales
@@ -526,7 +528,7 @@ QgsProfileSnapContext QgsElevationProfileCanvas::snapContext() const
   context.maximumPointDistanceDelta = 4 * xToleranceInPlotUnits;
   context.maximumPointElevationDelta = 4 * yToleranceInPlotUnits;
   context.displayRatioElevationVsDistance = ( ( mPlotItem->yMaximum() - mPlotItem->yMinimum() ) / ( mPlotItem->plotArea().height() ) )
-      / ( ( mPlotItem->xMaximum() - mPlotItem->xMinimum() ) * mPlotItem->mXScaleFactor / ( mPlotItem->plotArea().width() ) );
+                                            / ( ( mPlotItem->xMaximum() - mPlotItem->xMinimum() ) * mPlotItem->mXScaleFactor / ( mPlotItem->plotArea().width() ) );
 
   return context;
 }
@@ -543,7 +545,7 @@ QgsProfileIdentifyContext QgsElevationProfileCanvas::identifyContext() const
   context.maximumPointDistanceDelta = 4 * xToleranceInPlotUnits;
   context.maximumPointElevationDelta = 4 * yToleranceInPlotUnits;
   context.displayRatioElevationVsDistance = ( ( mPlotItem->yMaximum() - mPlotItem->yMinimum() ) / ( mPlotItem->plotArea().height() ) )
-      / ( ( mPlotItem->xMaximum() - mPlotItem->xMinimum() ) * mPlotItem->mXScaleFactor / ( mPlotItem->plotArea().width() ) );
+                                            / ( ( mPlotItem->xMaximum() - mPlotItem->xMinimum() ) * mPlotItem->mXScaleFactor / ( mPlotItem->plotArea().width() ) );
 
   context.project = mProject;
 
@@ -569,7 +571,7 @@ void QgsElevationProfileCanvas::setupLayerConnections( QgsMapLayer *layer, bool
   {
     case Qgis::LayerType::Vector:
     {
-      QgsVectorLayer *vl = qobject_cast< QgsVectorLayer * >( layer );
+      QgsVectorLayer *vl = qobject_cast<QgsVectorLayer *>( layer );
       if ( isDisconnect )
       {
         disconnect( vl, &QgsVectorLayer::featureAdded, this, &QgsElevationProfileCanvas::regenerateResultsForLayer );
@@ -876,15 +878,15 @@ void QgsElevationProfileCanvas::refresh()
   context.appendScope( QgsExpressionContextUtils::projectScope( mProject ) );
   request.setExpressionContext( context );
 
-  const QList< QgsMapLayer * > layersToGenerate = layers();
-  QList< QgsAbstractProfileSource * > sources;
-  const QList< QgsAbstractProfileSource * > registrySources = QgsApplication::profileSourceRegistry()->profileSources();
+  const QList<QgsMapLayer *> layersToGenerate = layers();
+  QList<QgsAbstractProfileSource *> sources;
+  const QList<QgsAbstractProfileSource *> registrySources = QgsApplication::profileSourceRegistry()->profileSources();
   sources.reserve( layersToGenerate.size() + registrySources.size() );
 
   sources << registrySources;
   for ( QgsMapLayer *layer : layersToGenerate )
   {
-    if ( QgsAbstractProfileSource *source = dynamic_cast< QgsAbstractProfileSource * >( layer ) )
+    if ( QgsAbstractProfileSource *source = dynamic_cast<QgsAbstractProfileSource *>( layer ) )
       sources.append( source );
   }
 
@@ -897,6 +899,11 @@ void QgsElevationProfileCanvas::refresh()
   generationContext.setMapUnitsPerDistancePixel( mProfileCurve->length() / mPlotItem->plotArea().width() );
   mCurrentJob->setContext( generationContext );
 
+  if ( mSubsectionsSymbol )
+  {
+    mCurrentJob->setSubsectionsSymbol( mSubsectionsSymbol->clone() );
+  }
+
   mCurrentJob->startGeneration();
   mPlotItem->setRenderer( mCurrentJob );
 
@@ -944,13 +951,13 @@ void QgsElevationProfileCanvas::onLayerProfileGenerationPropertyChanged()
   if ( !mCurrentJob || mCurrentJob->isActive() )
     return;
 
-  QgsMapLayerElevationProperties *properties = qobject_cast< QgsMapLayerElevationProperties * >( sender() );
+  QgsMapLayerElevationProperties *properties = qobject_cast<QgsMapLayerElevationProperties *>( sender() );
   if ( !properties )
     return;
 
-  if ( QgsMapLayer *layer = qobject_cast< QgsMapLayer * >( properties->parent() ) )
+  if ( QgsMapLayer *layer = qobject_cast<QgsMapLayer *>( properties->parent() ) )
   {
-    if ( QgsAbstractProfileSource *source = dynamic_cast< QgsAbstractProfileSource * >( layer ) )
+    if ( QgsAbstractProfileSource *source = dynamic_cast<QgsAbstractProfileSource *>( layer ) )
     {
       if ( mCurrentJob->invalidateResults( source ) )
         scheduleDeferredRegeneration();
@@ -964,13 +971,13 @@ void QgsElevationProfileCanvas::onLayerProfileRendererPropertyChanged()
   if ( !mCurrentJob || mCurrentJob->isActive() )
     return;
 
-  QgsMapLayerElevationProperties *properties = qobject_cast< QgsMapLayerElevationProperties * >( sender() );
+  QgsMapLayerElevationProperties *properties = qobject_cast<QgsMapLayerElevationProperties *>( sender() );
   if ( !properties )
     return;
 
-  if ( QgsMapLayer *layer = qobject_cast< QgsMapLayer * >( properties->parent() ) )
+  if ( QgsMapLayer *layer = qobject_cast<QgsMapLayer *>( properties->parent() ) )
   {
-    if ( QgsAbstractProfileSource *source = dynamic_cast< QgsAbstractProfileSource * >( layer ) )
+    if ( QgsAbstractProfileSource *source = dynamic_cast<QgsAbstractProfileSource *>( layer ) )
     {
       mCurrentJob->replaceSource( source );
     }
@@ -981,9 +988,12 @@ void QgsElevationProfileCanvas::onLayerProfileRendererPropertyChanged()
 
 void QgsElevationProfileCanvas::regenerateResultsForLayer()
 {
-  if ( QgsMapLayer *layer = qobject_cast< QgsMapLayer * >( sender() ) )
+  if ( !mCurrentJob )
+    return;
+
+  if ( QgsMapLayer *layer = qobject_cast<QgsMapLayer *>( sender() ) )
   {
-    if ( QgsAbstractProfileSource *source = dynamic_cast< QgsAbstractProfileSource * >( layer ) )
+    if ( QgsAbstractProfileSource *source = dynamic_cast<QgsAbstractProfileSource *>( layer ) )
     {
       if ( mCurrentJob->invalidateResults( source ) )
         scheduleDeferredRegeneration();
@@ -1051,11 +1061,9 @@ void QgsElevationProfileCanvas::refineResults()
 
     // for similar reasons we round the minimum distance off to multiples of the maximum error in map units
     const double distanceMin = std::floor( ( mPlotItem->xMinimum() * mPlotItem->mXScaleFactor - plotDistanceRange * 0.05 ) / context.maximumErrorMapUnits() ) * context.maximumErrorMapUnits();
-    context.setDistanceRange( QgsDoubleRange( std::max( 0.0, distanceMin ),
-                              mPlotItem->xMaximum() * mPlotItem->mXScaleFactor + plotDistanceRange * 0.05 ) );
+    context.setDistanceRange( QgsDoubleRange( std::max( 0.0, distanceMin ), mPlotItem->xMaximum() * mPlotItem->mXScaleFactor + plotDistanceRange * 0.05 ) );
 
-    context.setElevationRange( QgsDoubleRange( mPlotItem->yMinimum() - plotElevationRange * 0.05,
-                               mPlotItem->yMaximum() + plotElevationRange * 0.05 ) );
+    context.setElevationRange( QgsDoubleRange( mPlotItem->yMinimum() - plotElevationRange * 0.05, mPlotItem->yMaximum() + plotElevationRange * 0.05 ) );
     mCurrentJob->setContext( context );
   }
   scheduleDeferredRegeneration();
@@ -1072,17 +1080,17 @@ void QgsElevationProfileCanvas::updateChartFromPalette()
     mPlotItem->yAxis().setTextFormat( textFormat );
   }
   {
-    std::unique_ptr< QgsFillSymbol > chartFill( mPlotItem->chartBackgroundSymbol()->clone() );
+    std::unique_ptr<QgsFillSymbol> chartFill( mPlotItem->chartBackgroundSymbol()->clone() );
     chartFill->setColor( chartPalette.color( QPalette::ColorGroup::Active, QPalette::ColorRole::Window ) );
     mPlotItem->setChartBackgroundSymbol( chartFill.release() );
   }
   {
-    std::unique_ptr< QgsFillSymbol > chartBorder( mPlotItem->chartBorderSymbol()->clone() );
+    std::unique_ptr<QgsFillSymbol> chartBorder( mPlotItem->chartBorderSymbol()->clone() );
     chartBorder->setColor( chartPalette.color( QPalette::ColorGroup::Active, QPalette::ColorRole::Text ) );
     mPlotItem->setChartBorderSymbol( chartBorder.release() );
   }
   {
-    std::unique_ptr< QgsLineSymbol > chartMajorSymbol( mPlotItem->xAxis().gridMajorSymbol()->clone() );
+    std::unique_ptr<QgsLineSymbol> chartMajorSymbol( mPlotItem->xAxis().gridMajorSymbol()->clone() );
     QColor c = chartPalette.color( QPalette::ColorGroup::Active, QPalette::ColorRole::Text );
     c.setAlpha( 150 );
     chartMajorSymbol->setColor( c );
@@ -1090,7 +1098,7 @@ void QgsElevationProfileCanvas::updateChartFromPalette()
     mPlotItem->yAxis().setGridMajorSymbol( chartMajorSymbol.release() );
   }
   {
-    std::unique_ptr< QgsLineSymbol > chartMinorSymbol( mPlotItem->xAxis().gridMinorSymbol()->clone() );
+    std::unique_ptr<QgsLineSymbol> chartMinorSymbol( mPlotItem->xAxis().gridMinorSymbol()->clone() );
     QColor c = chartPalette.color( QPalette::ColorGroup::Active, QPalette::ColorRole::Text );
     c.setAlpha( 50 );
     chartMinorSymbol->setColor( c );
@@ -1153,11 +1161,10 @@ void QgsElevationProfileCanvas::setLayers( const QList<QgsMapLayer *> &layers )
 
   // filter list, removing null layers and invalid layers
   auto filteredList = layers;
-  filteredList.erase( std::remove_if( filteredList.begin(), filteredList.end(),
-                                      []( QgsMapLayer * layer )
-  {
-    return !layer || !layer->isValid();
-  } ), filteredList.end() );
+  filteredList.erase( std::remove_if( filteredList.begin(), filteredList.end(), []( QgsMapLayer *layer ) {
+                        return !layer || !layer->isValid();
+                      } ),
+                      filteredList.end() );
 
   mLayers = _qgis_listRawToQPointer( filteredList );
   for ( QgsMapLayer *layer : std::as_const( mLayers ) )
@@ -1218,7 +1225,7 @@ QgsPoint QgsElevationProfileCanvas::toMapCoordinates( const QgsPointXY &point )
   const double distanceAlongPlotPercent = dx / mPlotItem->plotArea().width();
   double distanceAlongCurveLength = distanceAlongPlotPercent * ( mPlotItem->xMaximum() - mPlotItem->xMinimum() ) * mPlotItem->mXScaleFactor + mPlotItem->xMinimum() * mPlotItem->mXScaleFactor;
 
-  std::unique_ptr< QgsPoint > mapXyPoint( mProfileCurve->interpolatePoint( distanceAlongCurveLength ) );
+  std::unique_ptr<QgsPoint> mapXyPoint( mProfileCurve->interpolatePoint( distanceAlongCurveLength ) );
   if ( !mapXyPoint )
     return QgsPoint();
 
@@ -1345,7 +1352,6 @@ const Qgs2DPlot &QgsElevationProfileCanvas::plot() const
 class QgsElevationProfilePlot : public Qgs2DPlot
 {
   public:
-
     QgsElevationProfilePlot( QgsProfilePlotRenderer *renderer )
       : mRenderer( renderer )
     {
@@ -1364,7 +1370,6 @@ class QgsElevationProfilePlot : public Qgs2DPlot
     double mXScale = 1;
 
   private:
-
     QgsProfilePlotRenderer *mRenderer = nullptr;
 };
 ///@endcond PRIVATE
@@ -1436,3 +1441,10 @@ void QgsElevationProfileCanvas::setSnappingEnabled( bool enabled )
 {
   mSnappingEnabled = enabled;
 }
+
+void QgsElevationProfileCanvas::setSubsectionsSymbol( QgsLineSymbol *symbol )
+{
+  mSubsectionsSymbol.reset( symbol );
+  std::unique_ptr<QgsLineSymbol> plotItemSymbol( mSubsectionsSymbol ? mSubsectionsSymbol->clone() : nullptr );
+  mPlotItem->setSubsectionsSymbol( plotItemSymbol.release() );
+}
