@@ -14,10 +14,12 @@
  ***************************************************************************/
 
 #include "qgsprocessingmultipleselectiondialog.h"
+#include "moc_qgsprocessingmultipleselectiondialog.cpp"
 #include "qgsgui.h"
 #include "qgssettings.h"
 #include "qgsfileutils.h"
 #include "qgsvectorlayer.h"
+#include "qgsmaplayerfactory.h"
 #include "qgsmeshlayer.h"
 #include "qgsrasterlayer.h"
 #include "qgspluginlayer.h"
@@ -38,17 +40,14 @@
 
 ///@cond NOT_STABLE
 
-QgsProcessingMultipleSelectionPanelWidget::QgsProcessingMultipleSelectionPanelWidget( const QVariantList &availableOptions,
-    const QVariantList &selectedOptions,
-    QWidget *parent )
+QgsProcessingMultipleSelectionPanelWidget::QgsProcessingMultipleSelectionPanelWidget( const QVariantList &availableOptions, const QVariantList &selectedOptions, QWidget *parent )
   : QgsPanelWidget( parent )
-  , mValueFormatter( []( const QVariant & v )->QString
-{
-  if ( v.userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>() )
-    return v.value< QgsProcessingModelChildParameterSource >().staticValue().toString();
-  else
-    return v.toString();
-} )
+  , mValueFormatter( []( const QVariant &v ) -> QString {
+    if ( v.userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>() )
+      return v.value<QgsProcessingModelChildParameterSource>().staticValue().toString();
+    else
+      return v.toString();
+  } )
 {
   setupUi( this );
 
@@ -67,8 +66,8 @@ QgsProcessingMultipleSelectionPanelWidget::QgsProcessingMultipleSelectionPanelWi
   mButtonToggleSelection = new QPushButton( tr( "Toggle Selection" ) );
   mButtonBox->addButton( mButtonToggleSelection, QDialogButtonBox::ActionRole );
 
-  connect( mButtonSelectAll, &QPushButton::clicked, this, [ = ] { selectAll( true ); } );
-  connect( mButtonClearSelection, &QPushButton::clicked, this, [ = ] { selectAll( false ); } );
+  connect( mButtonSelectAll, &QPushButton::clicked, this, [=] { selectAll( true ); } );
+  connect( mButtonClearSelection, &QPushButton::clicked, this, [=] { selectAll( false ); } );
   connect( mButtonToggleSelection, &QPushButton::clicked, this, &QgsProcessingMultipleSelectionPanelWidget::toggleSelection );
 
   connect( mButtonBox, &QDialogButtonBox::accepted, this, &QgsProcessingMultipleSelectionPanelWidget::acceptClicked );
@@ -183,8 +182,8 @@ void QgsProcessingMultipleSelectionPanelWidget::populateList( const QVariantList
   // we add selected options first, keeping the existing order of options
   for ( const QVariant &option : selectedOptions )
   {
-//    if isinstance(t, QgsProcessingModelChildParameterSource):
-//       item = QStandardItem(t.staticValue())
+    //    if isinstance(t, QgsProcessingModelChildParameterSource):
+    //       item = QStandardItem(t.staticValue())
     // else:
 
     addOption( option, mValueFormatter( option ), true );
@@ -199,7 +198,7 @@ void QgsProcessingMultipleSelectionPanelWidget::populateList( const QVariantList
   mSelectionList->setModel( mModel );
 }
 
-QList< int> QgsProcessingMultipleSelectionPanelWidget::existingMapLayerFromMimeData( const QMimeData *data ) const
+QList<int> QgsProcessingMultipleSelectionPanelWidget::existingMapLayerFromMimeData( const QMimeData *data ) const
 {
   const QgsMimeDataUtils::UriList uriList = QgsMimeDataUtils::decodeUriList( data );
   QList<int> indexes;
@@ -227,7 +226,7 @@ void QgsProcessingMultipleSelectionPanelWidget::dragEnterEvent( QDragEnterEvent
   if ( !( event->possibleActions() & Qt::CopyAction ) )
     return;
 
-  const QList< int> indexes = existingMapLayerFromMimeData( event->mimeData() );
+  const QList<int> indexes = existingMapLayerFromMimeData( event->mimeData() );
   if ( !indexes.isEmpty() )
   {
     // dragged an acceptable layer, phew
@@ -241,7 +240,7 @@ void QgsProcessingMultipleSelectionPanelWidget::dropEvent( QDropEvent *event )
   if ( !( event->possibleActions() & Qt::CopyAction ) )
     return;
 
-  const QList< int> indexes = existingMapLayerFromMimeData( event->mimeData() );
+  const QList<int> indexes = existingMapLayerFromMimeData( event->mimeData() );
   if ( !indexes.isEmpty() )
   {
     // dropped an acceptable layer, phew
@@ -262,12 +261,7 @@ void QgsProcessingMultipleSelectionPanelWidget::addOption( const QVariant &value
   // don't add duplicate options
   for ( int i = 0; i < mModel->rowCount(); ++i )
   {
-    if ( mModel->item( i )->data( Qt::UserRole ) == value ||
-         ( mModel->item( i )->data( Qt::UserRole ).userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>() &&
-           value.userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>() &&
-           mModel->item( i )->data( Qt::UserRole ).value< QgsProcessingModelChildParameterSource >() ==
-           value.value< QgsProcessingModelChildParameterSource >() )
-       )
+    if ( mModel->item( i )->data( Qt::UserRole ) == value || ( mModel->item( i )->data( Qt::UserRole ).userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>() && value.userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>() && mModel->item( i )->data( Qt::UserRole ).value<QgsProcessingModelChildParameterSource>() == value.value<QgsProcessingModelChildParameterSource>() ) )
     {
       if ( updateExistingTitle )
         mModel->item( i )->setText( title );
@@ -275,7 +269,7 @@ void QgsProcessingMultipleSelectionPanelWidget::addOption( const QVariant &value
     }
   }
 
-  std::unique_ptr< QStandardItem > item = std::make_unique< QStandardItem >( title );
+  auto item = std::make_unique<QStandardItem>( title );
   item->setData( value, Qt::UserRole );
   item->setCheckState( selected ? Qt::Checked : Qt::Unchecked );
   item->setCheckable( true );
@@ -288,7 +282,6 @@ void QgsProcessingMultipleSelectionPanelWidget::addOption( const QVariant &value
 //
 
 
-
 QgsProcessingMultipleSelectionDialog::QgsProcessingMultipleSelectionDialog( const QVariantList &availableOptions, const QVariantList &selectedOptions, QWidget *parent, Qt::WindowFlags flags )
   : QDialog( parent, flags )
 {
@@ -317,9 +310,7 @@ QVariantList QgsProcessingMultipleSelectionDialog::selectedOptions() const
 // QgsProcessingMultipleInputPanelWidget
 //
 
-QgsProcessingMultipleInputPanelWidget::QgsProcessingMultipleInputPanelWidget( const QgsProcessingParameterMultipleLayers *parameter, const QVariantList &selectedOptions,
-    const QList<QgsProcessingModelChildParameterSource> &modelSources,
-    QgsProcessingModelAlgorithm *model, QWidget *parent )
+QgsProcessingMultipleInputPanelWidget::QgsProcessingMultipleInputPanelWidget( const QgsProcessingParameterMultipleLayers *parameter, const QVariantList &selectedOptions, const QList<QgsProcessingModelChildParameterSource> &modelSources, QgsProcessingModelAlgorithm *model, QWidget *parent )
   : QgsProcessingMultipleSelectionPanelWidget( QVariantList(), selectedOptions, parent )
   , mParameter( parameter )
 {
@@ -349,7 +340,7 @@ void QgsProcessingMultipleInputPanelWidget::addFiles()
   QString path = settings.value( QStringLiteral( "/Processing/LastInputPath" ), QDir::homePath() ).toString();
 
   QString filter;
-  if ( const QgsFileFilterGenerator *generator = dynamic_cast< const QgsFileFilterGenerator * >( mParameter ) )
+  if ( const QgsFileFilterGenerator *generator = dynamic_cast<const QgsFileFilterGenerator *>( mParameter ) )
     filter = generator->createFileFilter();
   else
     filter = QObject::tr( "All files (*.*)" );
@@ -380,7 +371,7 @@ void QgsProcessingMultipleInputPanelWidget::addDirectory()
   settings.setValue( QStringLiteral( "/Processing/LastInputPath" ), dir );
 
   QStringList nameFilters;
-  if ( const QgsFileFilterGenerator *generator = dynamic_cast< const QgsFileFilterGenerator * >( mParameter ) )
+  if ( const QgsFileFilterGenerator *generator = dynamic_cast<const QgsFileFilterGenerator *>( mParameter ) )
   {
     const QStringList extensions = QgsFileUtils::extensionsFromFilter( generator->createFileFilter() );
     for ( const QString &extension : extensions )
@@ -397,15 +388,13 @@ void QgsProcessingMultipleInputPanelWidget::addDirectory()
     const QString fullPath = it.next();
     if ( fullPath.endsWith( QLatin1String( ".dbf" ), Qt::CaseInsensitive ) )
     {
-      if ( QFileInfo::exists( QStringLiteral( "%1.shp" ).arg( fullPath.chopped( 4 ) ) ) ||
-           QFileInfo::exists( QStringLiteral( "%1.SHP" ).arg( fullPath.chopped( 4 ) ) ) )
+      if ( QFileInfo::exists( QStringLiteral( "%1.shp" ).arg( fullPath.chopped( 4 ) ) ) || QFileInfo::exists( QStringLiteral( "%1.SHP" ).arg( fullPath.chopped( 4 ) ) ) )
       {
         // Skip DBFs that are sidecar files to a Shapefile
         continue;
       }
     }
-    else if ( fullPath.endsWith( QLatin1String( ".aux.xml" ), Qt::CaseInsensitive ) ||
-              fullPath.endsWith( QLatin1String( ".shp.xml" ), Qt::CaseInsensitive ) )
+    else if ( fullPath.endsWith( QLatin1String( ".aux.xml" ), Qt::CaseInsensitive ) || fullPath.endsWith( QLatin1String( ".shp.xml" ), Qt::CaseInsensitive ) )
     {
       // Skip XMLs that are sidecar files  to datasets
       continue;
@@ -415,7 +404,7 @@ void QgsProcessingMultipleInputPanelWidget::addDirectory()
   emit selectionChanged();
 }
 
-QList< int> QgsProcessingMultipleInputPanelWidget::existingMapLayerFromMimeData( const QMimeData *data, QgsMimeDataUtils::UriList &handledUrls ) const
+QList<int> QgsProcessingMultipleInputPanelWidget::existingMapLayerFromMimeData( const QMimeData *data, QgsMimeDataUtils::UriList &handledUrls ) const
 {
   handledUrls.clear();
   const QgsMimeDataUtils::UriList uriList = QgsMimeDataUtils::decodeUriList( data );
@@ -472,7 +461,7 @@ QStringList QgsProcessingMultipleInputPanelWidget::compatibleUrisFromMimeData( c
            || parameter->layerType() == Qgis::ProcessingSourceType::VectorLine
            || parameter->layerType() == Qgis::ProcessingSourceType::VectorPoint
            || parameter->layerType() == Qgis::ProcessingSourceType::VectorPolygon )
-         && u.layerType == QLatin1String( "vector" ) )
+         && u.layerType == QgsMapLayerFactory::typeToString( Qgis::LayerType::Vector ) )
     {
       bool acceptable = false;
       switch ( QgsWkbTypes::geometryType( u.wkbType ) )
@@ -505,16 +494,16 @@ QStringList QgsProcessingMultipleInputPanelWidget::compatibleUrisFromMimeData( c
         res.append( u.providerKey != QLatin1String( "ogr" ) ? QgsProcessingUtils::encodeProviderKeyAndUri( u.providerKey, u.uri ) : u.uri );
     }
     else if ( ( parameter->layerType() == Qgis::ProcessingSourceType::MapLayer || parameter->layerType() == Qgis::ProcessingSourceType::Raster )
-              && u.layerType == QLatin1String( "raster" ) && u.providerKey == QLatin1String( "gdal" ) )
+              && u.layerType == QgsMapLayerFactory::typeToString( Qgis::LayerType::Raster ) && u.providerKey == QLatin1String( "gdal" ) )
       res.append( u.uri );
-    else if ( ( parameter->layerType()  == Qgis::ProcessingSourceType::MapLayer || parameter->layerType() == Qgis::ProcessingSourceType::Mesh )
-              && u.layerType == QLatin1String( "mesh" ) && u.providerKey == QLatin1String( "mdal" ) )
+    else if ( ( parameter->layerType() == Qgis::ProcessingSourceType::MapLayer || parameter->layerType() == Qgis::ProcessingSourceType::Mesh )
+              && u.layerType == QgsMapLayerFactory::typeToString( Qgis::LayerType::Mesh ) && u.providerKey == QLatin1String( "mdal" ) )
       res.append( u.uri );
-    else if ( ( parameter->layerType()  == Qgis::ProcessingSourceType::MapLayer || parameter->layerType() == Qgis::ProcessingSourceType::PointCloud )
-              && u.layerType == QLatin1String( "pointcloud" ) )
+    else if ( ( parameter->layerType() == Qgis::ProcessingSourceType::MapLayer || parameter->layerType() == Qgis::ProcessingSourceType::PointCloud )
+              && u.layerType == QgsMapLayerFactory::typeToString( Qgis::LayerType::PointCloud ) )
       res.append( u.uri );
-    else if ( ( parameter->layerType()  == Qgis::ProcessingSourceType::MapLayer || parameter->layerType() == Qgis::ProcessingSourceType::VectorTile )
-              && u.layerType == QLatin1String( "vector-tile" ) )
+    else if ( ( parameter->layerType() == Qgis::ProcessingSourceType::MapLayer || parameter->layerType() == Qgis::ProcessingSourceType::VectorTile )
+              && u.layerType == QgsMapLayerFactory::typeToString( Qgis::LayerType::VectorTile ) )
       res.append( u.uri );
     // NOLINTEND(bugprone-branch-clone)
   }
@@ -525,11 +514,11 @@ QStringList QgsProcessingMultipleInputPanelWidget::compatibleUrisFromMimeData( c
   QStringList rawPaths;
   if ( data->hasUrls() )
   {
-    const QList< QUrl > urls = data->urls();
+    const QList<QUrl> urls = data->urls();
     rawPaths.reserve( urls.count() );
     for ( const QUrl &url : urls )
     {
-      const QString local =  url.toLocalFile();
+      const QString local = url.toLocalFile();
       if ( !rawPaths.contains( local ) )
         rawPaths.append( local );
     }
@@ -557,7 +546,7 @@ void QgsProcessingMultipleInputPanelWidget::dragEnterEvent( QDragEnterEvent *eve
 
   // maybe dragging layers from the project
   QgsMimeDataUtils::UriList handledUris;
-  const QList< int> indexes = existingMapLayerFromMimeData( event->mimeData(), handledUris );
+  const QList<int> indexes = existingMapLayerFromMimeData( event->mimeData(), handledUris );
   if ( !indexes.isEmpty() )
   {
     // dragged an acceptable layer, phew
@@ -582,7 +571,7 @@ void QgsProcessingMultipleInputPanelWidget::dropEvent( QDropEvent *event )
     return;
 
   QgsMimeDataUtils::UriList handledUris;
-  const QList< int> indexes = existingMapLayerFromMimeData( event->mimeData(), handledUris );
+  const QList<int> indexes = existingMapLayerFromMimeData( event->mimeData(), handledUris );
   if ( !indexes.isEmpty() )
   {
     // dropped an acceptable layer, phew
@@ -611,8 +600,7 @@ void QgsProcessingMultipleInputPanelWidget::dropEvent( QDropEvent *event )
 
 void QgsProcessingMultipleInputPanelWidget::populateFromProject( QgsProject *project )
 {
-  connect( project, &QgsProject::layerRemoved, this, [&]( const QString & layerId )
-  {
+  connect( project, &QgsProject::layerRemoved, this, [&]( const QString &layerId ) {
     for ( int i = 0; i < mModel->rowCount(); ++i )
     {
       const QStandardItem *item = mModel->item( i );
@@ -630,8 +618,7 @@ void QgsProcessingMultipleInputPanelWidget::populateFromProject( QgsProject *pro
   } );
 
   QgsSettings settings;
-  auto addLayer = [&]( const QgsMapLayer * layer )
-  {
+  auto addLayer = [&]( const QgsMapLayer *layer ) {
     const QString authid = layer->crs().authid();
     QString title;
     if ( settings.value( QStringLiteral( "Processing/Configuration/SHOW_CRS_DEF" ), true ).toBool() && !authid.isEmpty() )
@@ -735,7 +722,7 @@ void QgsProcessingMultipleInputPanelWidget::populateFromProject( QgsProject *pro
     case Qgis::ProcessingSourceType::Vector:
     case Qgis::ProcessingSourceType::VectorAnyGeometry:
     {
-      const QList<QgsVectorLayer *> options = QgsProcessingUtils::compatibleVectorLayers( project, QList< int >() << static_cast<int>( mParameter->layerType() ) );
+      const QList<QgsVectorLayer *> options = QgsProcessingUtils::compatibleVectorLayers( project, QList<int>() << static_cast<int>( mParameter->layerType() ) );
       for ( const QgsVectorLayer *layer : options )
       {
         addLayer( layer );
@@ -746,7 +733,7 @@ void QgsProcessingMultipleInputPanelWidget::populateFromProject( QgsProject *pro
 
     case Qgis::ProcessingSourceType::MapLayer:
     {
-      const QList<QgsVectorLayer *> vectors = QgsProcessingUtils::compatibleVectorLayers( project, QList< int >() );
+      const QList<QgsVectorLayer *> vectors = QgsProcessingUtils::compatibleVectorLayers( project, QList<int>() );
       for ( const QgsVectorLayer *layer : vectors )
       {
         addLayer( layer );
@@ -784,7 +771,7 @@ void QgsProcessingMultipleInputPanelWidget::populateFromProject( QgsProject *pro
     case Qgis::ProcessingSourceType::VectorLine:
     case Qgis::ProcessingSourceType::VectorPolygon:
     {
-      const QList<QgsVectorLayer *> vectors = QgsProcessingUtils::compatibleVectorLayers( project, QList< int >() << static_cast< int >( mParameter->layerType() ) );
+      const QList<QgsVectorLayer *> vectors = QgsProcessingUtils::compatibleVectorLayers( project, QList<int>() << static_cast<int>( mParameter->layerType() ) );
       for ( const QgsVectorLayer *layer : vectors )
       {
         addLayer( layer );
@@ -798,8 +785,7 @@ void QgsProcessingMultipleInputPanelWidget::populateFromProject( QgsProject *pro
 // QgsProcessingMultipleInputDialog
 //
 
-QgsProcessingMultipleInputDialog::QgsProcessingMultipleInputDialog( const QgsProcessingParameterMultipleLayers *parameter, const QVariantList &selectedOptions,
-    const QList< QgsProcessingModelChildParameterSource > &modelSources, QgsProcessingModelAlgorithm *model, QWidget *parent, Qt::WindowFlags flags )
+QgsProcessingMultipleInputDialog::QgsProcessingMultipleInputDialog( const QgsProcessingParameterMultipleLayers *parameter, const QVariantList &selectedOptions, const QList<QgsProcessingModelChildParameterSource> &modelSources, QgsProcessingModelAlgorithm *model, QWidget *parent, Qt::WindowFlags flags )
   : QDialog( parent, flags )
 {
   setWindowTitle( tr( "Multiple Selection" ) );
