@@ -23,6 +23,7 @@
 #include "qgsapplication.h"
 #include "qgsfeedback.h"
 #include "qgsvectorlayer.h"
+#include "qgsmessagelog.h"
 
 #include <QSqlRecord>
 #include <QSqlField>
@@ -95,7 +96,7 @@ QgsOracleProviderConnection::QgsOracleProviderConnection( const QString &uri, co
 
   if ( inputUri.hasParam( QStringLiteral( "estimatedMetadata" ) ) )
   {
-    currentUri.setUseEstimatedMetadata( inputUri.param( QStringLiteral( "estimatedMetadata" ) ) == QStringLiteral( "true" ) || inputUri.param( QStringLiteral( "estimatedMetadata" ) ) == '1' );
+    currentUri.setUseEstimatedMetadata( inputUri.param( QStringLiteral( "estimatedMetadata" ) ) == QLatin1String( "true" ) || inputUri.param( QStringLiteral( "estimatedMetadata" ) ) == '1' );
   }
 
   for ( const auto &param : EXTRA_CONNECTION_PARAMETERS )
@@ -165,6 +166,8 @@ QgsVectorLayer *QgsOracleProviderConnection::createSqlVectorLayer( const QgsAbst
     throw QgsProviderConnectionException( QObject::tr( "Could not create a SQL vector layer: SQL expression is empty." ) );
   }
 
+  const QString optionsSql { sanitizeSqlForQueryLayer( options.sql ) };
+
   QgsDataSourceUri tUri( uri() );
 
   tUri.setSql( options.filter );
@@ -173,25 +176,25 @@ QgsVectorLayer *QgsOracleProviderConnection::createSqlVectorLayer( const QgsAbst
   if ( !options.primaryKeyColumns.isEmpty() )
   {
     tUri.setKeyColumn( options.primaryKeyColumns.join( ',' ) );
-    tUri.setTable( QStringLiteral( "(%1)" ).arg( options.sql ) );
+    tUri.setTable( QStringLiteral( "(%1)" ).arg( optionsSql ) );
   }
   else
   {
     // Disable when there is no pk
     tUri.setUseEstimatedMetadata( false );
     int pkId { 0 };
-    while ( options.sql.contains( QStringLiteral( "qgis_generated_uid_%1_" ).arg( pkId ), Qt::CaseSensitivity::CaseInsensitive ) )
+    while ( optionsSql.contains( QStringLiteral( "qgis_generated_uid_%1_" ).arg( pkId ), Qt::CaseSensitivity::CaseInsensitive ) )
     {
       pkId++;
     }
     tUri.setKeyColumn( QStringLiteral( "qgis_generated_uid_%1_" ).arg( pkId ) );
 
     int sqlId { 0 };
-    while ( options.sql.contains( QStringLiteral( "qgis_generated_subq_%1_" ).arg( sqlId ), Qt::CaseSensitivity::CaseInsensitive ) )
+    while ( optionsSql.contains( QStringLiteral( "qgis_generated_subq_%1_" ).arg( sqlId ), Qt::CaseSensitivity::CaseInsensitive ) )
     {
       sqlId++;
     }
-    tUri.setTable( QStringLiteral( "(SELECT row_number() over (ORDER BY NULL) AS qgis_generated_uid_%1_, qgis_generated_subq_%3_.* FROM (%2\n) qgis_generated_subq_%3_\n)" ).arg( QString::number( pkId ), options.sql, QString::number( sqlId ) ) );
+    tUri.setTable( QStringLiteral( "(SELECT row_number() over (ORDER BY NULL) AS qgis_generated_uid_%1_, qgis_generated_subq_%3_.* FROM (%2\n) qgis_generated_subq_%3_\n)" ).arg( QString::number( pkId ), optionsSql, QString::number( sqlId ) ) );
   }
 
   if ( !options.geometryColumn.isEmpty() )
@@ -199,7 +202,7 @@ QgsVectorLayer *QgsOracleProviderConnection::createSqlVectorLayer( const QgsAbst
     tUri.setGeometryColumn( options.geometryColumn );
   }
 
-  std::unique_ptr<QgsVectorLayer> vl = std::make_unique<QgsVectorLayer>( tUri.uri( false ), options.layerName.isEmpty() ? QStringLiteral( "QueryLayer" ) : options.layerName, providerKey() );
+  auto vl = std::make_unique<QgsVectorLayer>( tUri.uri( false ), options.layerName.isEmpty() ? QStringLiteral( "QueryLayer" ) : options.layerName, providerKey() );
 
   // Try to guess the geometry and srid
   if ( !vl->isValid() )
@@ -212,7 +215,7 @@ QgsVectorLayer *QgsOracleProviderConnection::createSqlVectorLayer( const QgsAbst
             WHERE a.%1 IS NOT NULL %3
             ORDER BY a.%1.SDO_GTYPE
     )" )
-                          .arg( options.geometryColumn, options.sql, limit ) };
+                          .arg( options.geometryColumn, optionsSql, limit ) };
     const QList<QList<QVariant>> candidates { executeSql( sql ) };
     for ( const QList<QVariant> &row : std::as_const( candidates ) )
     {
@@ -287,6 +290,16 @@ QgsVectorLayer *QgsOracleProviderConnection::createSqlVectorLayer( const QgsAbst
   return vl.release();
 }
 
+Qgis::DatabaseProviderTableImportCapabilities QgsOracleProviderConnection::tableImportCapabilities() const
+{
+  return Qgis::DatabaseProviderTableImportCapability::SetGeometryColumnName | Qgis::DatabaseProviderTableImportCapability::SetPrimaryKeyName;
+}
+
+QString QgsOracleProviderConnection::defaultPrimaryKeyColumnName() const
+{
+  return QStringLiteral( "id" );
+}
+
 void QgsOracleProviderConnection::store( const QString &name ) const
 {
   QString baseKey = QStringLiteral( "/Oracle/connections/" );
@@ -313,6 +326,10 @@ void QgsOracleProviderConnection::store( const QString &name ) const
     {
       settings.setValue( param, dsUri.param( param ) );
     }
+    else
+    {
+      settings.remove( param );
+    }
   }
 
   // From configuration
@@ -322,6 +339,10 @@ void QgsOracleProviderConnection::store( const QString &name ) const
     {
       settings.setValue( p, configuration().value( p ) );
     }
+    else
+    {
+      settings.remove( p );
+    }
   }
   settings.endGroup();
   settings.endGroup();
@@ -347,6 +368,11 @@ QList<QgsVectorDataProvider::NativeType> QgsOracleProviderConnection::nativeType
   return types;
 }
 
+QString QgsOracleProviderConnection::defaultGeometryColumnName() const
+{
+  return QStringLiteral( "GEOM" );
+}
+
 QMultiMap<Qgis::SqlKeywordCategory, QStringList> QgsOracleProviderConnection::sqlDictionary()
 {
   return {
@@ -1349,7 +1375,7 @@ QgsAbstractDatabaseProviderConnection::QueryResult QgsOracleProviderConnection::
   if ( feedback && feedback->isCanceled() )
     return QgsAbstractDatabaseProviderConnection::QueryResult();
 
-  std::shared_ptr<QgsPoolOracleConn> pconn = std::make_shared<QgsPoolOracleConn>( QgsDataSourceUri { uri() }.connectionInfo( false ) );
+  auto pconn = std::make_shared<QgsPoolOracleConn>( QgsDataSourceUri { uri() }.connectionInfo( false ) );
   if ( !pconn->get() )
   {
     throw QgsProviderConnectionException( QObject::tr( "Connection failed: %1" ).arg( uri() ) );
@@ -1358,7 +1384,7 @@ QgsAbstractDatabaseProviderConnection::QueryResult QgsOracleProviderConnection::
   if ( feedback && feedback->isCanceled() )
     return QgsAbstractDatabaseProviderConnection::QueryResult();
 
-  std::unique_ptr<QgsOracleQuery> qry = std::make_unique<QgsOracleQuery>( pconn );
+  auto qry = std::make_unique<QgsOracleQuery>( pconn );
   std::chrono::steady_clock::time_point begin = std::chrono::steady_clock::now();
 
   QgsDatabaseQueryLogWrapper logWrapper { sql, uri(), providerKey(), QStringLiteral( "QgsAbstractDatabaseProviderConnection" ), QGS_QUERY_LOG_ORIGIN };
@@ -1449,6 +1475,7 @@ void QgsOracleProviderConnection::createVectorTable( const QString &schema, cons
   }
   QMap<int, int> map;
   QString errCause;
+  QString createdLayerUri;
   const Qgis::VectorExportResult res = QgsOracleProvider::createEmptyLayer(
     newUri.uri(),
     fields,
@@ -1456,6 +1483,7 @@ void QgsOracleProviderConnection::createVectorTable( const QString &schema, cons
     srs,
     overwrite,
     map,
+    createdLayerUri,
     errCause,
     options
   );
@@ -1463,6 +1491,26 @@ void QgsOracleProviderConnection::createVectorTable( const QString &schema, cons
     throw QgsProviderConnectionException( QObject::tr( "An error occurred while creating the vector layer: %1" ).arg( errCause ) );
 }
 
+QString QgsOracleProviderConnection::createVectorLayerExporterDestinationUri( const VectorLayerExporterOptions &options, QVariantMap &providerOptions ) const
+{
+  QgsDataSourceUri destUri( uri() );
+
+  destUri.setTable( options.layerName );
+  destUri.setSchema( options.schema );
+  destUri.setGeometryColumn( options.wkbType != Qgis::WkbType::NoGeometry ? ( options.geometryColumn.isEmpty() ? QStringLiteral( "GEOM" ) : options.geometryColumn ) : QString() );
+  if ( !options.primaryKeyColumns.isEmpty() )
+  {
+    if ( options.primaryKeyColumns.length() > 1 )
+    {
+      QgsMessageLog::logMessage( QStringLiteral( "Multiple primary keys are not supported by Oracle, ignoring" ), QString(), Qgis::MessageLevel::Info );
+    }
+    destUri.setKeyColumn( options.primaryKeyColumns.at( 0 ) );
+  }
+
+  providerOptions.clear();
+  return destUri.uri( false );
+}
+
 QString QgsOracleProviderConnection::tableUri( const QString &schema, const QString &name ) const
 {
   const auto tableInfo { table( schema, name ) };
